/*#include <sys/types.h>
*/
#include <unistd.h>
#include "libedid.h"
#include <stdio.h>
#include <string.h>

/* prints usage */
int usage(const char * exe_name);
/* strips out all audio flags and blocks from the CEA-861 extension block*/
int stripHdmi(edid_t*);
/* used in stripHdmi */
int stripHdmiCallback(
	edid_block_t* block,
	CEA861_BLKTYPE type,
	unsigned char* raw,
	int raw_len,
	void* user);


/* removes all extensions */
int disableAllExts(edid_t *edid);

int usage(const char *exe_name) {
	printf("Usage: %s [--to-dvi] input_file output_file\n", exe_name);
	printf(" --to-dvi Keeps the extensions but removes audio information and the HDMI\n");
	printf("          identifier. This should make an HDMI device appear to be DVI.\n");
	return 0;
}

int main(int argc, const char* argv[]) {
	edid_t edid;
	char invalid_reason[256];
	int mode;
	const char* input_file_name;
	const char* output_file_name;

	if (argc == 3) {
		mode = 0;
		input_file_name = argv[1];
		output_file_name = argv[2];
	} else if (argc == 4) {
		if (strcmp(argv[1], "--to-dvi") == 0) {
			mode = 1;
			input_file_name = argv[2];
			output_file_name = argv[3];
		} else {
			return usage(argv[0]);
		}
	} else {
		return usage(argv[0]);
	}

	memset(&edid, 0, sizeof(edid));
	if (!edidReadFile(&edid, input_file_name)) {
		return 0;
	}

	if (!edidIsValid(&edid, invalid_reason)) {
		printf("ERROR: Input EDID does not appear to be valid. (%s.)\n", invalid_reason);
		return 0;
	}

	if (mode == 0) {
		if (!disableAllExts(&edid)) {
			return 0;
		}
	} else {
		if (!stripHdmi(&edid)) {
			return 0;
		}
	}

	if (!edidIsValid(&edid, invalid_reason)) {
		printf("ERROR: Internal error, something inexplicable happened. (%s.)\n", invalid_reason);
		return 0;
	}

	if (!edidWriteFile(&edid, output_file_name)) {
		return 0;
	}

	return 1;
}

int stripHdmi(edid_t* edid) {
	int cea861_vers, change_made;
	edid_block_t * cea861;

	cea861 = edidFindExtensionBlock(edid, EDID_EXTTYPE_CEA861, NULL);
	if (!cea861) {
		printf("ERROR: CEA861 extension block is missing. A full 256 byte or larger edid file with CEA-861 extension is required for --strip-audio.\n");
		return 0;
	}

	cea861_vers = edidCEA861Version(cea861);
	if (cea861_vers < 2) {
		printf("ERROR: CEA861 extension block is older than version 2 and has no audio information.\n");
		return 0;
	}

	change_made	= edidCEA861ClearDTDFlag(cea861, CEA861_DTDFLAG_BASIC_AUDIO);
	if (cea861_vers == 2) {
		if (!change_made) {
			printf("ERROR: CEA861 is version 2 and basic audio flag already clear.\n");
			return 0;
		} else {
			/* version 2 does not have any audio, speaker, vendor blocks */
			return 1;
		}
	}

	change_made |= edidCEA861StripDataBlocks(cea861, stripHdmiCallback, NULL);
	if (!change_made) {
		printf("ERROR: CEA861 audio/hdmi information is already absent.\n");
		return 0;
	}

	edidFixChecksum(cea861);

	return 1;
}

int stripHdmiCallback(
	edid_block_t* block,
	CEA861_BLKTYPE type,
	unsigned char* raw,
	int raw_len,
	void* user) {

	static const unsigned char hdmi_header[] = {0x03, 0x0c, 0x00};

	switch (type) {
	case CEA861_BLKTYPE_VIDEO:    /* keep all video blocks */
		break;
	case CEA861_BLKTYPE_AUDIO:    /* strip all audio blocks */
	case CEA861_BLKTYPE_SPEAKERS: /* strip all speaker blocks */
		return 1;
	case CEA861_BLKTYPE_VENDOR:   /* strip the HDMI vendor block */
		return raw_len >= 3 && memcmp(hdmi_header, raw, 3)==0;
	}
	return 0;
}

int disableAllExts(edid_t *edid) {
	edid_block_t * primary;

	primary = edidBlock(edid, 0);
	if (!edidZeroExtensions(primary)) {
		printf("ERROR: Input EDID already has extensions flag cleared.\n");
		return 0;
	}
	edidFixChecksum(primary);
	edidTruncate(edid, 1);
	return 1;
}
