#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <string.h>

#include "libedid.h"

void edidInit(edid_t* edid, int block_count) {
	int i;

	edid->block_count = block_count;
	edid->blocks = (edid_block_t**) malloc(block_count * sizeof(edid_block_t*));
	for (i=0; i<block_count; i++) {
		edid->blocks[i] = (edid_block_t*) calloc(1, sizeof(edid_block_t));
	}
}

void edidDestroy(edid_t* edid) {
	int i;
	for (i=0; i<edid->block_count; i++) {
		free(edid->blocks[i]);
	}
	free(edid->blocks);
}

edid_block_t* edidBlock(edid_t* edid, int index) {
	return edid->blocks[index];
}


int edidReadFile(edid_t* edid, const char* filename) {
	FILE* input;
	struct stat stats;
	int i;

	if (stat(filename, &stats)) {
		printf("ERROR: Input file \"%s\" could not be opened.\n", filename);
		return 0;
	}

	if (stats.st_size % EDID_BLOCK_SIZE != 0) {
		printf("ERROR: Input file \"%s\" has a bad filesize (not a multiple of %d)\n", filename, EDID_BLOCK_SIZE);
		return 0;
	}

	input = fopen(filename, "rb");
	if (input == NULL) {
		printf("ERROR: Input file \"%s\" could not be opened.\n", filename);
		return 0;
	}

	edidInit(edid, stats.st_size / EDID_BLOCK_SIZE);

	for (i=0; i<edid->block_count; i++) {
		if (EDID_BLOCK_SIZE != fread(edidBlock(edid, i)->data, 1, EDID_BLOCK_SIZE, input)) {
			printf("ERROR: Input file \"%s\" could not be read.\n", filename);
			fclose(input);
			return 0;
		}
	}

	fclose(input);
	return 1;
}



int edidWriteFile(edid_t* edid, const char* filename) {
	FILE* output;
	int blk;

	output = fopen(filename, "wb");
	if (output == NULL) {
		printf("ERROR: Output file \"%s\" could not be opened.\n", filename);
		return 0;
	}

	for (blk=0; blk<edid->block_count; blk++) {
		if (EDID_BLOCK_SIZE != fwrite(edidBlock(edid, blk)->data, 1, EDID_BLOCK_SIZE, output)) {
			printf("ERROR: Output file \"%s\" could not be written.\n", filename);
			fclose(output);
			return 0;
		}
	}

	fclose(output);
	return 1;
}

int edidIsValid(edid_t* edid, char reason[]) {
	int i;

	/* must have at least one block */
	if (edid->block_count <= 0) {
		strcpy(reason, "EDID must have at least one block");
		return 0;
	}

	/* first block must have a special header */
	if (!edidFirstBlockHeaderValid(edidBlock(edid, 0))) {
		strcpy(reason, "First block has an invalid header");
		return 0;
	}

	/* every block must have a correct checksum */
	for (i=0; i<edid->block_count; i++) {
		if (!edidChecksumValid(edidBlock(edid, i))) {
			sprintf(reason, "Block number %d has an invalid checksum", i);
			return 0;
		}
	}

	return 1;
}

int edidFirstBlockHeaderValid(edid_block_t* block) {
	static const unsigned char header[8] = {
		0x00, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0x00
	};

	return memcmp(block->data, header, 8) == 0;
}

int edidChecksumValid(edid_block_t* block) {
	int total, i;

	total = 0;
	for (i=0; i<EDID_BLOCK_SIZE; i++) {
		total += block->data[i];
	}

	return (total & 0xFF) == 0;
}

void edidFixChecksum(edid_block_t* block) {
	int i, total;

	total = 0;
	for (i=0; i<EDID_BLOCK_SIZE-1; i++) {
		total += block->data[i];
	}
	block->data[EDID_BLOCK_SIZE-1] = (-total) & 0xFF;
}

int edidZeroExtensions(edid_block_t* block) {
	int old_edid;

	old_edid = block->data[EDID_EXT_BYTE];
	block->data[EDID_EXT_BYTE] = 0;
	return old_edid;
}

void edidCopyBlock(edid_block_t* dest, const edid_block_t* src) {
	if (src == dest) {
		return;
	}

	memcpy(&dest->data[0], &src->data[0], EDID_BLOCK_SIZE);
}


int edidTruncate(edid_t* edid, int block_count) {
	int old_count, i;
	old_count = edid->block_count;

	if (old_count > block_count) {
		for (i=block_count; i<old_count; i++) {
			free(edid->blocks[i]);
		}
	}
	/* we dont reallocate the block pointer array, but its pretty
	insignficant */
	edid->block_count = block_count;

	return old_count;
}

edid_block_t* edidFindExtensionBlock(edid_t* edid, EDID_EXTTYPE extension, int * block_index) {
	int blk;
	edid_block_t * cea861;

	for (blk=1; blk<edid->block_count; blk++) {
		cea861 = edidBlock(edid, blk);
		if (cea861->data[EDID_EXT_HEADER_BYTE] == extension) {
			if (block_index) {
				*block_index = blk;
			}
			return cea861;
		}
	}
	return NULL;
}

int edidCEA861ClearDTDFlag(edid_block_t *cea861, CEA861_DTDFLAG flag) {
	if (cea861->data[CEA861_DTDFLAGS_BYTE] & flag) {
		cea861->data[CEA861_DTDFLAGS_BYTE] ^=  flag;
		return 1;
	}
	return 0;
}

int edidCEA861StripDataBlocks(edid_block_t *cea861,
							  CEA861_DATABLKCALLBACK callback,
							  void* user) {
	int data_ends, cur_addr, block_size, block_type, shift, blocks_stripped;

	blocks_stripped = 0;

	/* the data blocks end where the DTD's start */
	data_ends = cea861->data[CEA861_DTDSTARTADDR_BYTE];
	cur_addr = CEA861_DATABLKSTART_BYTE;
	while (cur_addr < data_ends)  { /* loop over all blocks */
		/* bottom 5 bits, +1 for cur addr byte */
		block_size = (cea861->data[cur_addr] & 0x1F) + 1;
		/* top 3 bits */
		block_type = cea861->data[cur_addr] & 0xE0;

		if (cur_addr + block_size > CEA861_PADEND_BYTE) {
			printf("ERROR: CEA861 block table address out of bounds.\n");
			/* todo: handle this better */
			return 0;
		}

		/* ask the user if we want to delete this block */
		if (callback(cea861, block_type, &cea861->data[cur_addr+1], block_size-1, user)) {
			/* calculate the number of bytes we have to shift down from the top */
			shift = CEA861_PADEND_BYTE - (cur_addr + block_size);
			/* shift the other blocks down */
			memmove(&cea861->data[cur_addr], &cea861->data[cur_addr + block_size], shift);
			/* we shifted all the blocks down, so zero out the top bytes */
			memset(&cea861->data[cur_addr + shift], block_size, 0);
			blocks_stripped++;
			data_ends -= block_size;
		} else {
			/* only advance when we havnt shifted new data into place */
			cur_addr += block_size;
		}
	}
	/* write back out the new endpoint of the data (since we deleted some) */
	cea861->data[CEA861_DTDSTARTADDR_BYTE] = data_ends;

	return blocks_stripped;	
}



int edidCEA861Version(edid_block_t *cea861) {
	return cea861->data[CEA861_VERSION_BYTE];
}
