#ifndef LIBEDID_H
#define LIBEDID_H

/* num bytes in an edid block */
#define EDID_BLOCK_SIZE 128

typedef struct {
	unsigned char data[EDID_BLOCK_SIZE];
} edid_block_t;

typedef struct {
	int block_count;
	edid_block_t** blocks;
} edid_t;

/* address of extension count */
#define EDID_EXT_BYTE 126
/* in an extended block, the 0th byte contains the block-type header */
#define EDID_EXT_HEADER_BYTE     0
typedef enum EDID_EXTTYPE_tag {
	/* extended block type CEA861 (doc missing, using wikipedia) */
	EDID_EXTTYPE_CEA861 = 0x02,
	/* video timing block (doc: VTBextRa.pdf) */
	EDID_EXTTYPE_VTB    = 0x10,
	/* depricated EDID 2.0 extension */
	/*EDID_EXTTYPE_EDID2  = 0x20,*/
	/* display information (doc: DIEXT.pdf) */
	EDID_EXTTYPE_DIEXT  = 0x40,
	/* localized strings (doc: LSExtRa.pdf) */
	EDID_EXTTYPE_LS     = 0x50,
	/* digital packet video link */
	EDID_EXTTYPE_DPVL   = 0x60,
	/* "extension block map" */
	EDID_EXTTYPE_BLKMAP = 0xF0,
	/* manufacturer extension */
	EDID_EXTTYPE_MANUF  = 0xFF
} EDID_EXTTYPE;

/* makes sure we are looking at a valid edid */
int edidIsValid(edid_t* edid, char invalid_reason[]);
/* zeroes the extensions, returns old extension flag value */
int edidZeroExtensions(edid_block_t* block);
/* tests if the checksum is valid */
int edidChecksumValid(edid_block_t* block);
/* tests if the edid header looks right */
int edidFirstBlockHeaderValid(edid_block_t* block);
/* Fixes the checksum (last byte) */
void edidFixChecksum(edid_block_t* block);
/* reads in an edid from file */
int edidReadFile(edid_t* edid, const char* filename);
/* writes out an edid to file */
int edidWriteFile(edid_t* edid, const char* filename);
/* truncates the edid to a particular size */
int edidTruncate(edid_t* edid, int block_count);
/* scans through the extra blocks looking for a specific extension */
edid_block_t* edidFindExtensionBlock(edid_t* edid, EDID_EXTTYPE extension, int * block_index);
/* convenience function to get a particular block, returns old size */
edid_block_t* edidBlock(edid_t* edid, int index);
/* copies one edid block over another */
void edidCopyBlock(edid_block_t* dest, const edid_block_t* src);

#define CEA861_VERSION_BYTE      1
#define CEA861_DTDSTARTADDR_BYTE 2
#define CEA861_DTDFLAGS_BYTE     3
#define CEA861_DATABLKSTART_BYTE 4
#define CEA861_PADEND_BYTE       126

typedef enum CEA861_DTDFLAG_tag {
	CEA861_DTDFLAG_YCbCr422        = (1 << 4),
	CEA861_DTDFLAG_YCbCr444        = (1 << 5),
	CEA861_DTDFLAG_BASIC_AUDIO     = (1 << 6),
	CEA861_DTDFLAG_BASIC_UNDERSCAN = (1 << 7)
} CEA861_DTDFLAG;

typedef enum CEA861_BLKTYPE_tag {
	CEA861_BLKTYPE_AUDIO    = (1<<5),
	CEA861_BLKTYPE_VIDEO    = (2<<5),
	CEA861_BLKTYPE_VENDOR   = (3<<5),
	CEA861_BLKTYPE_SPEAKERS = (4<<5)
} CEA861_BLKTYPE;

/* gets the version number of the CEA-861 block */
int edidCEA861Version(edid_block_t *cea861);
/* clear the specified flag from the DTDFlags area */
int edidCEA861ClearDTDFlag(edid_block_t *cea861, CEA861_DTDFLAG flag);

typedef int (*CEA861_DATABLKCALLBACK)(
	edid_block_t* block,
	CEA861_BLKTYPE type,
	unsigned char* raw,
	int raw_len,
	void* user);

/* remove datablocks based on callback return value */
int edidCEA861StripDataBlocks(edid_block_t *cea861,
							  CEA861_DATABLKCALLBACK callback,
							  void* user);
							  


#endif /* LIBEDID_H */
